/*global window, rJS, JSHINT, RSVP, jIO, document, loopEventListener */
/*jslint nomen: true, indent: 2, maxerr: 3*/
(function (window, rJS, JSHINT, jIO, loopEventListener) {
  "use strict";

  function getURLContentAsText(gadget, url) {
    return new RSVP.Queue()
      .push(function () {
        return gadget.crib_sw_get(url);
      })
      .push(function (data) {
        var type;
        type = data.type;
        return jIO.util.readBlobAsText(data, type);
      })
      .push(function (event) {
        return event.target.result;
      });
  }

  function displayURLList(gadget, event) {
    var url_list = [];
    return new RSVP.Queue()
    .push(function () {
        return gadget.crib_sw_allDocs();
      })
    .push(function (data) {
      var promise_list = [],
        tmp_url_list = [],
        url;
      if (data.hasOwnProperty("urls")) {
        tmp_url_list = data.urls;
      } else {
        tmp_url_list = data;
      }
      for (url in tmp_url_list) {
        if (tmp_url_list.hasOwnProperty(url)) {
          if (url.endsWith(".js")) {
            url_list.push(url);
            promise_list.push(RSVP.all([
              gadget.getUrlFor({page: 'editor', url: url}),
              getURLContentAsText(gadget, url)
            ]));
          }
        }
      }
      return RSVP.all(promise_list);
    })
    .push(function (result_list) {
      var contentsElement = gadget.props.element.querySelector('.crib-url-list-content tbody'),
        footer_element = gadget.props.element.querySelector('.crib-url-list-content tfoot'),
        url_number, error_number = 0,
        url, trElement, tdElement;
      // Clear out the existing items from the list.
      while (contentsElement.firstChild) {
        contentsElement.removeChild(contentsElement.firstChild);
      }
      // Add each cached URL to the list, one by one.
      for (url_number = 0; url_number < url_list.length; url_number++) {
        url = url_list[url_number];
        var element;
        JSHINT(result_list[url_number][1]);
        trElement = document.createElement('tr');
        tdElement = document.createElement('td');
        tdElement.innerHTML = (JSHINT.data().errors ? "<span>&#x274C;</span>" : "");
        trElement.appendChild(tdElement);
        tdElement = document.createElement('td');
        element = document.createElement('a');
        element.setAttribute('href', result_list[url_number][0]);
        element.textContent = url;
        tdElement.appendChild(element);
        trElement.appendChild(tdElement);
        tdElement = document.createElement('td');
        element = document.createElement('a');
        element.textContent = "Go";
        element.setAttribute('href', url);
        element.setAttribute('target', "_blank");
        element.setAttribute("class", "btn btn-primary btn-xs");
        tdElement.appendChild(element);
        trElement.appendChild(tdElement);
        contentsElement.appendChild(trElement);
        error_number = JSHINT.data().errors ? error_number + 1 : error_number;
      }
      while (footer_element.firstChild) {
        footer_element.removeChild(footer_element.firstChild);
      }
      trElement = document.createElement('tr');
      tdElement = document.createElement('td');
      tdElement.innerHTML = "<span>" + error_number + " &#x274C;</span>";
      trElement.appendChild(tdElement);
      tdElement = document.createElement('td');
      tdElement.innerHTML =  url_number + " URLs";
      trElement.appendChild(tdElement);
      tdElement = document.createElement('td');
      trElement.appendChild(tdElement);
      footer_element.appendChild(trElement);
    });
  }

  rJS(window)
    .ready(function (g) {
      g.props = {};
      return g.getElement()
        .push(function (element) {
          g.props.element = element;
          g.props.start_deferred = RSVP.defer();
        });
    })
    .declareAcquiredMethod("crib_sw_allDocs", "crib_sw_allDocs")
    .declareAcquiredMethod("crib_sw_get", "crib_sw_get")
    .declareAcquiredMethod("getUrlFor", "getUrlFor")
    .declareMethod('render', function (options) {
      var gadget = this;
      if (options === undefined)
        options = {};
      gadget.props.options = options;
      return new RSVP.Queue()
        .push(function () {
          return displayURLList(gadget, undefined);
        })
        .push(function () {
          return gadget.props.start_deferred.resolve();
        });
    })
    .declareService(function () {
      var gadget = this;
      return new RSVP.Queue()
        .push(function () {
          return gadget.props.start_deferred.promise;
        })
        .push(function () {
          return loopEventListener(
            gadget.props.element.querySelector(".crib-url-list-content"),
            'submit',
            false,
            function (event) {displayURLList(gadget, event); }
          );
        });
    });
}(window, rJS, JSHINT, jIO, loopEventListener));